/* eslint-disable @typescript-eslint/no-unused-expressions */
import { Form, Button, Col, Row, Input, Typography, Grid, message, Space } from 'antd';
import React, { useState, useEffect, useRef, useMemo } from 'react';
import styles from './index.less';
import ChatFirstContent from './components/content';
import {
  LoadingOutlined,
  MenuOutlined,
  PlusOutlined,
  SettingOutlined,
  UserOutlined,
} from '@ant-design/icons';
import classNames from 'classnames';
import { history } from 'umi';
import LoginModal from '@/components/LoginModal';
import NewSession from '@/components/NewSession';
import { useWebSocket, useRequest } from 'ahooks';
import { GetChatGLMToken, GetPromptList } from '@/services/chatGLM/api';
import { useAccess } from 'umi';
import { PageLoading } from '@ant-design/pro-layout';
import { checkIsTokenExpires } from '@/access';

const { useBreakpoint } = Grid;
const { Title } = Typography;
const { Item } = Form;

const isMobileDevice = (): boolean => {
  const ua = navigator.userAgent;
  return /Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test(ua);
};

interface IHistory {
  prompt: string;
  answer: string;
}

const Chat: React.FC = () => {
  const screens = useBreakpoint();
  const isMobile = screens.xs;
  const [chat, setChat] = useState('');
  const [showSider, setShowSider] = useState(!isMobileDevice());
  const [firstChat, setFirstChat] = useState(false);
  const [form] = Form.useForm();
  const { data: glmToken, run: getChatGLMToken } = useRequest(GetChatGLMToken, { manual: true });
  const {
    data: promptList,
    loading: promptLoading,
    run: getPromptList,
  } = useRequest(GetPromptList, { manual: true });

  const [answer, setAnswer] = useState('');
  const [loading, setLoading] = useState(true);
  const [loginModalOpen, setLoginModalOpen] = useState(false);
  const access = useAccess();

  const messageHistory = useRef<IHistory[]>([]);
  const { sendMessage } = useWebSocket('wss://glm-api.baibaomen.com/ws', {
    onMessage: (event) => {
      const text = event.data;
      console.log(text);
      if (text == '{{BBMCPLT}}') {
        setLoading(false);
      } else {
        setAnswer(text);
        window.scrollTo(0, document.body.scrollHeight);
        console.log(text);
      }
    },
    onOpen: () => {
      console.log('连接成功！');
      setLoading(false);
    },
    onError: () => {
      setLoading(false);
    },
  });

  messageHistory.current = useMemo(() => {
    if (messageHistory.current.length > 0)
      messageHistory.current[messageHistory.current.length - 1].answer += answer;
    return messageHistory.current;
  }, [answer]);

  async function handleSendMessage() {
    if (!checkIsTokenExpires()) {
      messageHistory.current.push({ prompt: chat, answer: '' });
      setLoading(true);
      sendMessage &&
        glmToken &&
        sendMessage(
          JSON.stringify({
            prompt: chat,
            max_length: 2048,
            top_p: 0.7,
            temperature: 0.95,
            history: [messageHistory.current.map((h) => h.prompt)],
            token: glmToken,
          }),
        );
    }
  }

  useEffect(() => {
    if (access?.token) {
      getChatGLMToken();
      getPromptList();
      setLoginModalOpen(false);
    } else {
      setLoginModalOpen(true);
    }
  }, [access]);

  const onSend = () => {
    form.setFieldValue('chatInput', '');
    setFirstChat(true);
    handleSendMessage();
  };

  const setInput = (value: any) => {
    setChat(value?.firstInput);
    form.setFieldValue('chatInput', value?.firstInput);
  };

  const onChange = (value: any) => {
    value?.chatInput && setChat(value?.chatInput);
    value?.firstInput && setInput(value);
  };

  const renderMobileHeader = () => {
    return (
      <Row align={'middle'} justify="space-between" gutter={1} className={styles.mobileHeader}>
        <Col push={1}>
          <MenuOutlined
            onClick={() => {
              setShowSider(!showSider);
            }}
          />
        </Col>
        <Col>长沙振升集团</Col>
        <Col pull={1} />
      </Row>
    );
  };

  return (
    <>
      <LoginModal open={loginModalOpen} setOpen={setLoginModalOpen} />
      {isMobile && renderMobileHeader()}
      <Row>
        <Col className={`${styles.siderWrapper}`}>
          <div
            style={{ top: isMobile && '44px', zIndex: isMobile ? 13 : 10 }}
            className={`${styles.sider} ${showSider ? styles.siderOpen : ''}`}
          >
            <div className={styles.siderChatContent}>
              <div style={{ padding: '30px 20px 0 20px' }}>
                <Button size="large" block type="primary" icon={<PlusOutlined />}>
                  新建对话
                </Button>
              </div>
            </div>
            <Row
              className={styles.chatFooter}
              style={{ width: '240px', marginBottom: isMobile ? 80 : 60 }}
              justify={'center'}
              align={'middle'}
            >
              <Col
                span={24}
                className={styles.chatFooterItem}
                hidden={!access.roles?.find((x) => ['管理员', 'admin'].includes(x))}
                onClick={() => {
                  history.push('/');
                }}
              >
                <Space wrap>
                  <SettingOutlined />
                  后台管理
                </Space>
              </Col>
              <Col span={24} className={styles.chatFooterItem} style={{ color: '#009B95' }}>
                <Space wrap>
                  <UserOutlined />
                  {access?.name || '请先登录'}
                </Space>
              </Col>
            </Row>
          </div>
        </Col>
        <Form
          form={form}
          initialValues={{ chatInput: chat }}
          name="chat"
          onValuesChange={onChange}
          onFinish={onSend}
        >
          <Col
            className={styles.chatContent}
            style={{
              marginLeft: isMobile ? 0 : 240,
              width: isMobile ? 'calc(100vw)' : 'calc(100vw - 240px)',
            }}
          >
            <Row justify={'center'}>
              <Col span={'24'}>
                {!isMobile && (
                  <div className={styles.header} hidden={firstChat}>
                    <Row>
                      <Col span={24}>
                        <Title level={2} style={{ marginBottom: 8, fontSize: 32 }}>
                          长沙振升集团
                        </Title>
                      </Col>
                      <Col span={24}>
                        <Title level={5} className={styles.subTitle}>
                          AI智能小助手
                        </Title>
                      </Col>
                    </Row>
                  </div>
                )}
              </Col>
              {promptLoading && (
                <Col span={24}>
                  <PageLoading />
                </Col>
              )}
              <Col span={'24'} style={{ padding: isMobile ? '' : '0 100px' }}>
                {firstChat ? (
                  <div style={{ marginBottom: 150 }}>
                    {messageHistory.current.map((item, index) => (
                      <NewSession
                        key={'new' + index}
                        questionValue={item.prompt}
                        answerValue={item.answer}
                      />
                    ))}
                  </div>
                ) : (
                  <div className={classNames(isMobile && styles.mobileContent)}>
                    <Item name="firstInput">
                      <ChatFirstContent list={promptList?.items || []} />
                    </Item>
                  </div>
                )}
              </Col>

              <div
                className={styles.footer}
                style={{
                  marginLeft: isMobile ? 0 : '240px',
                  height: isMobile ? '100px' : '120px',
                  width: isMobile ? '100vw' : 'calc(100vw - 240px)',
                }}
              >
                <Row justify={'center'} align={'middle'} style={{ textAlign: 'center' }}>
                  <Col span={24} style={{ textAlign: 'center' }}>
                    <Space style={{ textAlign: 'center' }}>
                      <div
                        className={styles.chatWrapper}
                        style={{
                          marginLeft: firstChat && !isMobile ? '82px' : 0,
                          width: isMobile
                            ? 'calc(100vw - 108px)'
                            : firstChat
                            ? 'calc(100vw - 482px)'
                            : 'calc(100vw - 550px)',
                        }}
                      >
                        <Item name="chatInput">
                          <Input
                            autoComplete={false}
                            required
                            placeholder="在这里输入你想知道的问题，给我几句提示，我来帮你回答"
                            size="large"
                          />
                        </Item>
                      </div>
                      <div className={styles.outputButtonWrapper}>
                        <Item name="outputButton">
                          <Button htmlType="submit" type="primary" size="large" disabled={loading}>
                            {loading ? (
                              <>
                                &nbsp;
                                <LoadingOutlined />
                                &nbsp; &nbsp;
                              </>
                            ) : (
                              '发送'
                            )}
                          </Button>
                        </Item>
                      </div>
                    </Space>
                  </Col>
                </Row>
              </div>
            </Row>
          </Col>
        </Form>
      </Row>
    </>
  );
};

export default Chat;
